#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QLabel>
#include <QString>
#include <QMainWindow>

#include "types.h"
#include "setcwd.h"
#include "database.h"
#include "mapviewdlg.h"
#include "operatorid.h"
#include "qcustomplot.h"
#include "filterviewdlg.h"
#include "configuredlg.h"
#include "logviewfilter.h"
#include "eventlistmodel.h"
#include "timeintervaldlg.h"
#include "selectcolumnsdlg.h"
#include "eventlistviewfilter.h"


namespace Ui {
   class MainWindow;
}

class MainWindow : public QMainWindow
{
   Q_OBJECT

public:
   friend class database;
   explicit MainWindow(QWidget *parent = 0);
   ~MainWindow();
   QString GetAppVersion();            // Returns program version.
   QString GetAppBuildDate();          // Returns program build date.

protected:
   void closeEvent(QCloseEvent*);      // Handles File|Exit and close button.

signals:
   // Signals for the log view.
   void WriteLog(logmodel::msg_type, QString);
   void WriteLog2File();
   void ClearLog();

public slots:
   // Slots for the main window.
   void appStarting();
   void SetWindowTitle();

   // Slots for the 'File' menu.
   void Open_DefaultDatabase();
   void Open_Database();
   void Open_LocalDatabase();
   void Open_CatalogFile();
   void Open_IndexFile();
   void Open_LastUsedDatabase();
   void Close_Database();
   void Edit_SeisanDef();
   void Edit_MulpltDef();
   bool SetTimeInterval();
   void SetWorkDir();
   void SetConfiguration();

   // Slots for the 'Event List' menu.
   void SetFilter();
   void RemoveFilter();
   void SelectColumns();
   void ResizeColumns();

   // Slots for the 'Functions' menu.
   void PoissonDistribution();
   void Gumble3DistributionPlot();
   void GutenbergRichterRelation();
   void CompletenessCheck();
   void WeichertMethod();
   void SetEventIndicator();
   void SetModelIndicator();
   void SimpleWaveformViewer();
   void MagnitudeVsMagnitude();
   void EventsPerYear();
   void TimeOfDayDistribution();
   void DriftOfHypocenters();
   void Export2CSV();
   void TimeOfDayVsDistance();
   void TimeOfDayVsTimeOfYear();
   void Sample_a();
   void Sample_b();

   // Slots for the 'Help' menu.
   void AboutSeisanExplorer();

   // Slot for the Tab widget.
   void CurrentTabChanged(int);

   // Slots for the status bar.
   void SetStatusBarOperator();
   void SetStatusBarTimeInterval();

   // Slot for database events.
   void DbOpenProgress(db::dbStatus, int);
   void DbFilterProgress(char);
   void DbOpenResult(db::dbStatus, db::dbError);

   // Slots for the Event List View widget.
   void CCMR_EventListView(const QPoint&);
   void ELV_Associate_Events();
   void ELV_Copy_Events();
   void ELV_Delete_Event();
   void ELV_Edit_StationHyp();
   void ELV_Duplicate_Event();
   void ELV_Edit_Comment_Lines();
   void ELV_Edit_With_Text_Editor();
   void ELV_Launch_EEV();
   void ELV_Reload_Event();
   void ELV_Load_Event();
   void ELV_Unload_Event();
   void ELV_Locate_Event();
   void ELV_Merge_Events();
   void ELV_Plot_With_Mulplot();
   void ELV_Plot_With_Mulplot_Menu();
   void ELV_Refresh_View();
   void ELV_Register_Event();
   void ELV_Set_Distance_Indicator();
   void ELV_Show_Events_With_GE();
   void ELV_Show_Events_With_GM();
   void ELV_Show_Events_With_SV();
   void ELV_GoToRowNr();
   void ELV_GoToNextNewEvent();
   void ELV_Select_All();
   void ELV_SortIndicatorChanged(int, Qt::SortOrder);
   void ELV_EditEvent(const QModelIndex&);

   // Slot for the Log View widget.
   void CCMR_LogView(const QPoint&);
   void LV_Edit_Event();
   void LV_Edit_LogFile();
   void LV_AutoSaveLog();

private:
   Ui::MainWindow *ui;

   // Objects related to name, version and build date.
   QString AppName;                        // Application name.
   QString AppVersion;                     // Application version.
   QString AppBuildDate;                   // Application build date.

   // Objects related to application status.
   app_status AppStatus;                   // Current operating status of application.
   bool OpenLastUsed;
   bool CatFileOpened;                     // 'True' if a catalog file is currently open as a local database.
   QString CatalogFile;                    // Full path to open catalog file.
   bool DoOnceAtLoadStart;
   OperatorId Operator;                    // Instance of the operator identity class.
   QLabel LblOperator;                     // This label is used to show operator id in the status bar.
   QLabel LblInterval;                     // This label is used to show time interval in the status bar.
   SetCWD WD;                              // Class for handling the setting of current working directory.
   void LoadProgramSettings();             // Load program settings.
   void SaveProgramSettings();             // Save program settings.

   // Objects related to seisan database and S-files.
   db DB;                                  // Instance of the database class used to hold seisan database.
   QString SEISAN_TOP;                     // Value of SEISAN_TOP environment variable.
   QString DEF_BASE;                       // Value of DEF_BASE environment variable (default database).
   QString DatabasePath;                   // Path to database or path/filename to index file. Including db name.
   QString DatabaseName;                   // Name of database. Path excluded.
   QString LastUsedDatabase;               // Name of last used database, or local database directory, or index file.
   db_type LastUsedDbType;                 // Type of database last opened.
   db_type CurrentDbType;                  // Type of database currrently open.
   db::dbError dberror;                    // Used to check return values from database class functions.
   bool NoTimeIntQuery;                    // If true when opening a database, do not ask user for time interval.
   bool NoCloseQuery;                      // If true when closing a database, do not ask user for confirmation.


   // Objects related to the event list view/model.
   QColor Color;
   QPalette Palette;
   QListEventID ELVSelection;              // Holds list of selected rows in Event List View.
   QHeaderView *ELVHeader;                 // Pointer to Event List View widget's header.
   eventlistmodel *ELVModel;               // Pointer to the model for the Event List View.
   void SetEventListViewHeaders();         // This function hides or shows columns in then event list.
   eventlistviewfilter *ELVFilter;         // Pointer to event filter for the Event List View.

   // Objects related to the log widget/model.
   logmodel *LogModel;                     // Pointer to the log model.
   logviewfilter *LVFilter;                // Pointer to event filter for the Log View.

   // Objects related to the map view dialog.
   MapViewDlg *MapView;                    // Pointer to the map view dialog.

   // Objects related to time interval settings.
   TimeIntervalDlg *TimeIntDlg;            // Pointer to the 'Set Time Interval' dialog.

   // Objects related to database filtering.
   FilterViewDlg *FilterDlg;
   db_filter DbFilter;                     // Holds the database filter configuration.

   // Objects related to the 'Seclect Columns' dialog box.
   SelectColumnsDlg *SelectColDlg;

   // Objects related to the 'Configure' dialog box.
   app_config Config;                      // Structure holding program configuration data.
   ConfigureDlg *ConfigDlg;                // Pointer to the 'Explorer Configuration' dialog.

   // Other objects.
   QString Message;                        // Used to hold messages.
   char str11[12];                         // Used for different things.
   char ConsFile[80];                      // File used by seisan library functions.
   int AccTimeInt;                         // Used with associate event function.
   void Plot_With_Mulplot(bool);           // Used by the ELV_Plot_With_Mulplot slots.
   void EditParamFile(char, char);         // Function for editing Seisan parameter files.

   // These functions are all defined
   // in the file 'mainwindowlib.cpp'.
   bool DeleteEvent(int, int*, bool);      // Deletes an event file.
   void CleanUpOnExit();
   bool WriteAllEvents2CatFile(QString);   // Writes all events to a catalog file.
};

#endif // MAINWINDOW_H
